/* -----------------------------------------------------------------------------
 * Copyright (c) 2016 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        05. July 2016
 * $Revision:    V1.00
 *
 * Project:      Flash Programming Functions for Kinetis KV1x
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */

#include <stdint.h>
#include "..\FlashOS.H"        /* FlashOS Structures */
#include "fsl_flash.h"


#define STACK_SIZE   1024      /* Stack Size */


#define XACC1_IDX    4
#define XACC2_IDX   12
#define SACC1_IDX   20
#define SACC2_IDX   28

uint8_t        flash_FSEC_FOPT[SECURITY_SIZE];
flash_config_t flash_config;

/* This function initializes all the RO, RW data in the PI library.
   It is called from the C library startup code.
   The flash algorithm does not execute the C library startup code
   so we must call it manually.
 */
extern void __cpp_initialize__aeabi_(void);

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
  int ret;

  __cpp_initialize__aeabi_();

  /* initialize unsecure key */
  #if (SECURITY_SIZE == 4)
    flash_FSEC_FOPT[0] = 0xFEU;      /* FSEC */
    flash_FSEC_FOPT[1] = 0xFFU;      /* FOPT */
    flash_FSEC_FOPT[2] = 0xFFU;
    flash_FSEC_FOPT[3] = 0xFFU;
  #else
    flash_FSEC_FOPT[0] = 0xFFU;
    flash_FSEC_FOPT[1] = 0xFFU;
    flash_FSEC_FOPT[2] = 0xFFU;
    flash_FSEC_FOPT[3] = 0xFFU;
    flash_FSEC_FOPT[4] = 0xFEU;      /* FSEC */
    flash_FSEC_FOPT[5] = 0xFFU;      /* FOPT */
    flash_FSEC_FOPT[6] = 0xFFU;
    flash_FSEC_FOPT[7] = 0xFFU;
  #endif

  /* set run mode (remove low power mode) */
  RunMode_Set();

  /* disable the watchdog */
  Watchdog_Disable();

  /* initialize global flash properties structure */
  ret = FLASH_Init(&flash_config);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  return (0);                                                 /* Finished without Errors */
}


/*  Blank Check Block in Flash Memory
 *    Parameter:      adr:  Block Start Address
 *                    sz:   Block Size (in bytes)
 *                    pat:  Block Pattern
 *    Return Value:   0 - OK,  1 - Failed
 */
#ifndef FAC
int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {
  int ret;

  ret = FLASH_VerifyErase(&flash_config, adr, sz, kFLASH_MarginValueNormal);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
#else
int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {

  return (0);                                                 /* treat as erased */
}
#endif


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */
#ifndef FAC
int EraseChip (void) {
  status_t ret;

  ret = FLASH_EraseAll(&flash_config, kFLASH_ApiEraseKey);
  if (ret != kStatus_FLASH_Success)
  {
    return (1);
  }

  /* program FSEC FOPT bytes */
  ret = FLASH_Program(&flash_config, SECURITY_LOCATION, (uint32_t *)flash_FSEC_FOPT, SECURITY_SIZE);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
#endif


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */
#ifndef FAC
int EraseSector (unsigned long adr) {
  int ret;

  ret = FLASH_Erase(&flash_config, adr, FSL_FEATURE_FLASH_PFLASH_BLOCK_SECTOR_SIZE,   kFLASH_ApiEraseKey);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
#else
int EraseSector (unsigned long adr) {

  return (0);                                                 /* treat as always OK */
}
#endif


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */
#ifndef FAC
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  status_t ret;

  sz = (sz + (FSL_FEATURE_FLASH_PFLASH_BLOCK_WRITE_UNIT_SIZE - 1)) & ~(FSL_FEATURE_FLASH_PFLASH_BLOCK_WRITE_UNIT_SIZE - 1);     /* Align to PGM_SIZE_BYTE */
//  sz = ALIGN_UP(sz, FSL_FEATURE_FLASH_PFLASH_BLOCK_WRITE_UNIT_SIZE);     /* Align to write size */

  ret = FLASH_Program(&flash_config, adr, (uint32_t *)buf, sz);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
#else
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  int ret;

  if (buf[0] != 0)   /* program XACC1 ? */
  {
    ret = FLASH_ProgramOnce(&flash_config, 0x10, (uint32_t *)&buf[XACC1_IDX], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (1);                                                 /* Finished with Errors */
    }
  }

  if (buf[1] != 0)   /* program XACC2 ? */
  {
    ret = FLASH_ProgramOnce(&flash_config, 0x11, (uint32_t *)&buf[XACC2_IDX], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (1);                                                 /* Finished with Errors */
    }
  }

  if (buf[2] != 0)   /* program SACC1 ? */
  {
    ret = FLASH_ProgramOnce(&flash_config, 0x12, (uint32_t *)&buf[SACC1_IDX], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (1);                                                 /* Finished with Errors */
    }
  }

  if (buf[3] != 0)   /* program SACC2 ? */
  {
    ret = FLASH_ProgramOnce(&flash_config, 0x13, (uint32_t *)&buf[SACC2_IDX], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (1);                                                 /* Finished with Errors */
    }
  }

  return (0);                                                     /* Finished without Errors */
}
#endif


/*
 *  Verify Flash Contents
 *    Parameter:      adr:  Start Address
 *                    sz:   Size (in bytes)
 *                    buf:  Data
 *    Return Value:   (adr+sz) - OK, Failed Address
 */
#ifndef FAC
#else
unsigned long Verify (unsigned long adr, unsigned long sz, unsigned char *buf) {
  uint32_t ret;
  uint8_t  data[8];

  if (buf[0] != 0)
  {
    ret = FLASH_ReadOnce(&flash_config, 0x10, (uint32_t *)&data[0], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (adr);                                               /* Finished with Errors */
    }
    if (*((uint64_t *)&data[0]) != *((uint64_t *)&buf[XACC1_IDX]))
    {
      return (adr + XACC1_IDX);                                   /* Finished with Errors */
    }
  }

  if (buf[1] !=0 )
  {
    ret = FLASH_ReadOnce(&flash_config, 0x11, (uint32_t *)&data[0], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (adr);                                               /* Finished with Errors */
    }
    if (*((uint64_t *)&data[0]) != *((uint64_t *)&buf[XACC2_IDX]))
    {
      return (adr + XACC2_IDX);                                   /* Finished with Errors */
    }
  }

  if (buf[2] !=0 )
  {
    ret = FLASH_ReadOnce(&flash_config, 0x12, (uint32_t *)&data[0], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (adr);                                               /* Finished with Errors */
    }
    if (*((uint64_t *)&data[0]) != *((uint64_t *)&buf[SACC1_IDX]))
    {
      return (adr + SACC1_IDX);                                   /* Finished with Errors */
    }
  }

  if (buf[3] !=0 )
  {
    ret = FLASH_ReadOnce(&flash_config, 0x13, (uint32_t *)&data[0], 8);
    if (ret != kStatus_FLASH_Success)
    {
      return (adr);                                               /* Finished with Errors */
    }
    if (*((uint64_t *)&data[0]) != *((uint64_t *)&buf[SACC2_IDX]))
    {
      return (adr + SACC2_IDX);                                   /* Finished with Errors */
    }
  }

  return (adr + sz);                                              /* Finished without Errors */
}
#endif
